(* DO NOT EDIT: automatically generated by kernel/genOpcodeFiles.ml *)
val opACC0 : int

val opACC1 : int

val opACC2 : int

val opACC3 : int

val opACC4 : int

val opACC5 : int

val opACC6 : int

val opACC7 : int

val opACC : int

val opPUSH : int

val opPUSHACC1 : int

val opPUSHACC2 : int

val opPUSHACC3 : int

val opPUSHACC4 : int

val opPUSHACC5 : int

val opPUSHACC6 : int

val opPUSHACC7 : int

val opPUSHACC : int

val opPUSHACCMANY : int

val opPOP : int

val opENVACC0 : int

val opENVACC1 : int

val opENVACC2 : int

val opENVACC3 : int

val opENVACC : int

val opPUSHENVACC0 : int

val opPUSHENVACC1 : int

val opPUSHENVACC2 : int

val opPUSHENVACC3 : int

val opPUSHENVACC : int

val opPUSHENVACCMANY : int

val opPUSH_RETADDR : int

val opAPPLY : int

val opAPPLY1 : int

val opAPPLY2 : int

val opAPPLY3 : int

val opAPPLY4 : int

val opAPPTERM : int

val opAPPTERM1 : int

val opAPPTERM2 : int

val opAPPTERM3 : int

val opRETURN : int

val opRESTART : int

val opGRAB : int

val opGRABREC : int

val opCLOSURE : int

val opCLOSUREREC : int

val opCLOSURECOFIX : int

val opOFFSETCLOSURE0 : int

val opOFFSETCLOSURE1 : int

val opOFFSETCLOSURE : int

val opPUSHOFFSETCLOSURE0 : int

val opPUSHOFFSETCLOSURE1 : int

val opPUSHOFFSETCLOSURE : int

val opGETGLOBAL : int

val opPUSHGETGLOBAL : int

val opMAKEBLOCK : int

val opMAKEBLOCK1 : int

val opMAKEBLOCK2 : int

val opMAKEBLOCK3 : int

val opMAKEBLOCK4 : int

val opSWITCH : int

val opPUSHFIELDS : int

val opGETFIELD0 : int

val opGETFIELD1 : int

val opGETFIELD : int

val opSETFIELD : int

val opPROJ : int

val opENSURESTACKCAPACITY : int

val opCONST0 : int

val opCONST1 : int

val opCONST2 : int

val opCONST3 : int

val opCONSTINT : int

val opPUSHCONST0 : int

val opPUSHCONST1 : int

val opPUSHCONST2 : int

val opPUSHCONST3 : int

val opPUSHCONSTINT : int

val opACCUMULATE : int

val opMAKESWITCHBLOCK : int

val opMAKEACCU : int

val opSUBSTINSTANCE : int

val opBRANCH : int

val opCHECKADDINT63 : int

val opCHECKADDCINT63 : int

val opCHECKADDCARRYCINT63 : int

val opCHECKSUBINT63 : int

val opCHECKSUBCINT63 : int

val opCHECKSUBCARRYCINT63 : int

val opCHECKMULINT63 : int

val opCHECKMULCINT63 : int

val opCHECKDIVINT63 : int

val opCHECKMODINT63 : int

val opCHECKDIVSINT63 : int

val opCHECKMODSINT63 : int

val opCHECKDIVEUCLINT63 : int

val opCHECKDIV21INT63 : int

val opCHECKLXORINT63 : int

val opCHECKLORINT63 : int

val opCHECKLANDINT63 : int

val opCHECKLSLINT63 : int

val opCHECKLSRINT63 : int

val opCHECKASRINT63 : int

val opCHECKADDMULDIVINT63 : int

val opCHECKEQINT63 : int

val opCHECKLTINT63 : int

val opCHECKLEINT63 : int

val opCHECKLTSINT63 : int

val opCHECKLESINT63 : int

val opCHECKCOMPAREINT63 : int

val opCHECKCOMPARESINT63 : int

val opCHECKHEAD0INT63 : int

val opCHECKTAIL0INT63 : int

val opCHECKOPPFLOAT : int

val opCHECKABSFLOAT : int

val opCHECKEQFLOAT : int

val opCHECKLTFLOAT : int

val opCHECKLEFLOAT : int

val opCHECKCOMPAREFLOAT : int

val opCHECKEQUALFLOAT : int

val opCHECKCLASSIFYFLOAT : int

val opCHECKADDFLOAT : int

val opCHECKSUBFLOAT : int

val opCHECKMULFLOAT : int

val opCHECKDIVFLOAT : int

val opCHECKSQRTFLOAT : int

val opCHECKFLOATOFINT63 : int

val opCHECKFLOATNORMFRMANTISSA : int

val opCHECKFRSHIFTEXP : int

val opCHECKLDSHIFTEXP : int

val opCHECKNEXTUPFLOAT : int

val opCHECKNEXTDOWNFLOAT : int

val opCHECKNEXTUPFLOATINPLACE : int

val opCHECKNEXTDOWNFLOATINPLACE : int

val opCHECKCAMLCALL2_1 : int

val opCHECKCAMLCALL1 : int

val opCHECKCAMLCALL2 : int

val opCHECKCAMLCALL3 : int

val opCHECKCAMLCALL3_1 : int

val opSTOP : int

