(* generated by dune *)

(** @canonical Coqdeplib.Args *)
module Args = Coqdeplib__Args

(** @canonical Coqdeplib.Common *)
module Common = Coqdeplib__Common

(** @canonical Coqdeplib.Dep_info *)
module Dep_info = Coqdeplib__Dep_info

(** @canonical Coqdeplib.Error *)
module Error = Coqdeplib__Error

(** @canonical Coqdeplib.File_util *)
module File_util = Coqdeplib__File_util

(** @canonical Coqdeplib.Fl *)
module Fl = Coqdeplib__Fl

(** @canonical Coqdeplib.Lexer *)
module Lexer = Coqdeplib__Lexer

(** @canonical Coqdeplib.Loadpath *)
module Loadpath = Coqdeplib__Loadpath

(** @canonical Coqdeplib.Makefile *)
module Makefile = Coqdeplib__Makefile

(** @canonical Coqdeplib.Rocqdep_main *)
module Rocqdep_main = Coqdeplib__Rocqdep_main

(** @canonical Coqdeplib.Static_toplevel_libs *)
module Static_toplevel_libs = Coqdeplib__Static_toplevel_libs
