(* generated by dune *)

(** @canonical Tuto3_plugin.Construction_game *)
module Construction_game = Tuto3_plugin__Construction_game

(** @canonical Tuto3_plugin.G_tuto3 *)
module G_tuto3 = Tuto3_plugin__G_tuto3

(** @canonical Tuto3_plugin.Tuto_tactic *)
module Tuto_tactic = Tuto3_plugin__Tuto_tactic
