(* generated by dune *)

(** @canonical Ring_plugin.G_ring *)
module G_ring = Ring_plugin__G_ring

(** @canonical Ring_plugin.Ring *)
module Ring = Ring_plugin__Ring

(** @canonical Ring_plugin.Ring_ast *)
module Ring_ast = Ring_plugin__Ring_ast
