
# 11 "plugins/rtauto/g_rtauto.mlg"
 

open Ltac_plugin


# 9 "plugins/rtauto/g_rtauto.ml"

let _ = Mltop.add_known_module "rocq-runtime.plugins.rtauto"
let () = Tacentries.tactic_extend "rocq-runtime.plugins.rtauto" "rtauto" ~level:0 
         [(Tacentries.TyML (Tacentries.TyIdent ("rtauto", Tacentries.TyNil), 
           (fun ist -> 
# 20 "plugins/rtauto/g_rtauto.mlg"
                    Refl_tauto.rtauto_tac 
# 17 "plugins/rtauto/g_rtauto.ml"
)))]

