(* generated by dune *)

(** @canonical Firstorder_core_plugin.Formula *)
module Formula = Firstorder_core_plugin__Formula

(** @canonical Firstorder_core_plugin.Ground *)
module Ground = Firstorder_core_plugin__Ground

(** @canonical Firstorder_core_plugin.Instances *)
module Instances = Firstorder_core_plugin__Instances

(** @canonical Firstorder_core_plugin.Rules *)
module Rules = Firstorder_core_plugin__Rules

(** @canonical Firstorder_core_plugin.Sequent *)
module Sequent = Firstorder_core_plugin__Sequent

(** @canonical Firstorder_core_plugin.Unify *)
module Unify = Firstorder_core_plugin__Unify
