
# 11 "plugins/btauto/g_btauto.mlg"
 

open Ltac_plugin


# 9 "plugins/btauto/g_btauto.ml"

let _ = Mltop.add_known_module "rocq-runtime.plugins.btauto"
let () = Tacentries.tactic_extend "rocq-runtime.plugins.btauto" "btauto" ~level:0 
         [(Tacentries.TyML (Tacentries.TyIdent ("btauto", Tacentries.TyNil), 
           (fun ist -> 
# 20 "plugins/btauto/g_btauto.mlg"
                    Refl_btauto.Btauto.tac 
# 17 "plugins/btauto/g_btauto.ml"
)))]

