(* generated by dune *)

(** @canonical Tuto1_plugin.G_tuto1 *)
module G_tuto1 = Tuto1_plugin__G_tuto1

(** @canonical Tuto1_plugin.Inspector *)
module Inspector = Tuto1_plugin__Inspector

(** @canonical Tuto1_plugin.Simple_check *)
module Simple_check = Tuto1_plugin__Simple_check

(** @canonical Tuto1_plugin.Simple_declare *)
module Simple_declare = Tuto1_plugin__Simple_declare

(** @canonical Tuto1_plugin.Simple_print *)
module Simple_print = Tuto1_plugin__Simple_print
