(* generated by dune *)

(** @canonical Ltac2_plugin.G_ltac2 *)
module G_ltac2 = Ltac2_plugin__G_ltac2

(** @canonical Ltac2_plugin.Tac2bt *)
module Tac2bt = Ltac2_plugin__Tac2bt

(** @canonical Ltac2_plugin.Tac2core *)
module Tac2core = Ltac2_plugin__Tac2core

(** @canonical Ltac2_plugin.Tac2dyn *)
module Tac2dyn = Ltac2_plugin__Tac2dyn

(** @canonical Ltac2_plugin.Tac2entries *)
module Tac2entries = Ltac2_plugin__Tac2entries

(** @canonical Ltac2_plugin.Tac2env *)
module Tac2env = Ltac2_plugin__Tac2env

(** @canonical Ltac2_plugin.Tac2expr *)
module Tac2expr = Ltac2_plugin__Tac2expr

(** @canonical Ltac2_plugin.Tac2externals *)
module Tac2externals = Ltac2_plugin__Tac2externals

(** @canonical Ltac2_plugin.Tac2extffi *)
module Tac2extffi = Ltac2_plugin__Tac2extffi

(** @canonical Ltac2_plugin.Tac2ffi *)
module Tac2ffi = Ltac2_plugin__Tac2ffi

(** @canonical Ltac2_plugin.Tac2intern *)
module Tac2intern = Ltac2_plugin__Tac2intern

(** @canonical Ltac2_plugin.Tac2interp *)
module Tac2interp = Ltac2_plugin__Tac2interp

(** @canonical Ltac2_plugin.Tac2match *)
module Tac2match = Ltac2_plugin__Tac2match

(** @canonical Ltac2_plugin.Tac2print *)
module Tac2print = Ltac2_plugin__Tac2print

(** @canonical Ltac2_plugin.Tac2qexpr *)
module Tac2qexpr = Ltac2_plugin__Tac2qexpr

(** @canonical Ltac2_plugin.Tac2quote *)
module Tac2quote = Ltac2_plugin__Tac2quote

(** @canonical Ltac2_plugin.Tac2stdlib *)
module Tac2stdlib = Ltac2_plugin__Tac2stdlib

(** @canonical Ltac2_plugin.Tac2tactics *)
module Tac2tactics = Ltac2_plugin__Tac2tactics

(** @canonical Ltac2_plugin.Tac2types *)
module Tac2types = Ltac2_plugin__Tac2types

(** @canonical Ltac2_plugin.Tac2typing_env *)
module Tac2typing_env = Ltac2_plugin__Tac2typing_env

(** @canonical Ltac2_plugin.Tac2val *)
module Tac2val = Ltac2_plugin__Tac2val
