(* generated by dune *)

(** @canonical Cc_core_plugin.Ccalgo *)
module Ccalgo = Cc_core_plugin__Ccalgo

(** @canonical Cc_core_plugin.Ccprojectability *)
module Ccprojectability = Cc_core_plugin__Ccprojectability

(** @canonical Cc_core_plugin.Ccproof *)
module Ccproof = Cc_core_plugin__Ccproof

(** @canonical Cc_core_plugin.Cctac *)
module Cctac = Cc_core_plugin__Cctac
