(* generated by dune *)

(** @canonical Rtauto_plugin.G_rtauto *)
module G_rtauto = Rtauto_plugin__G_rtauto

(** @canonical Rtauto_plugin.Proof_search *)
module Proof_search = Rtauto_plugin__Proof_search

(** @canonical Rtauto_plugin.Refl_tauto *)
module Refl_tauto = Rtauto_plugin__Refl_tauto
