
# 11 "plugins/nsatz/g_nsatz.mlg"
 

open Ltac_plugin
open Stdarg


# 10 "plugins/nsatz/g_nsatz.ml"

let _ = Mltop.add_known_module "rocq-runtime.plugins.nsatz"
let () = Tacentries.tactic_extend "rocq-runtime.plugins.nsatz" "nsatz_compute" ~level:0 
         [(Tacentries.TyML (Tacentries.TyIdent ("nsatz_compute", Tacentries.TyArg (
                                                                 Extend.TUentry (Genarg.get_arg_tag wit_constr), 
                                                                 Tacentries.TyNil)), 
           (fun lt ist -> 
# 21 "plugins/nsatz/g_nsatz.mlg"
                                       Nsatz.nsatz_compute (EConstr.Unsafe.to_constr lt) 
# 20 "plugins/nsatz/g_nsatz.ml"
)))]

