(* generated by dune *)

(** @canonical Tuto2_plugin.Counter *)
module Counter = Tuto2_plugin__Counter

(** @canonical Tuto2_plugin.Custom *)
module Custom = Tuto2_plugin__Custom

(** @canonical Tuto2_plugin.G_tuto2 *)
module G_tuto2 = Tuto2_plugin__G_tuto2

(** @canonical Tuto2_plugin.Persistent_counter *)
module Persistent_counter = Tuto2_plugin__Persistent_counter
