(* generated by dune *)

(** @canonical Gramlib.Gramext *)
module Gramext = Gramlib__Gramext

(** @canonical Gramlib.Grammar *)
module Grammar = Gramlib__Grammar

(** @canonical Gramlib.LStream *)
module LStream = Gramlib__LStream

(** @canonical Gramlib.Plexing *)
module Plexing = Gramlib__Plexing

(** @canonical Gramlib.Stream *)
module Stream = Gramlib__Stream
