
# 17 "plugins/ltac/g_eqdecide.mlg"
 

open Eqdecide
open Stdarg


# 10 "plugins/ltac/g_eqdecide.ml"

let _ = Mltop.add_known_module "rocq-runtime.plugins.ltac"
let () = Tacentries.tactic_extend "rocq-runtime.plugins.ltac" "decide_equality" ~level:0 
         [(Tacentries.TyML (Tacentries.TyIdent ("decide", Tacentries.TyIdent ("equality", 
                                                          Tacentries.TyNil)), 
           (fun ist -> 
# 27 "plugins/ltac/g_eqdecide.mlg"
                               decideEqualityGoal 
# 19 "plugins/ltac/g_eqdecide.ml"
)))]

let () = Tacentries.tactic_extend "rocq-runtime.plugins.ltac" "compare" ~level:0 
         [(Tacentries.TyML (Tacentries.TyIdent ("compare", Tacentries.TyArg (
                                                           Extend.TUentry (Genarg.get_arg_tag wit_constr), 
                                                           Tacentries.TyArg (
                                                           Extend.TUentry (Genarg.get_arg_tag wit_constr), 
                                                           Tacentries.TyNil))), 
           (fun c1 c2 ist -> 
# 31 "plugins/ltac/g_eqdecide.mlg"
                                           compare c1 c2 
# 31 "plugins/ltac/g_eqdecide.ml"
)))]

