(* generated by dune *)

(** @canonical Btauto_plugin.G_btauto *)
module G_btauto = Btauto_plugin__G_btauto

(** @canonical Btauto_plugin.Refl_btauto *)
module Refl_btauto = Btauto_plugin__Refl_btauto
