(* generated by dune *)

(** @canonical Ltac2_ltac1_plugin.G_ltac2_ltac1 *)
module G_ltac2_ltac1 = Ltac2_ltac1_plugin__G_ltac2_ltac1

(** @canonical Ltac2_ltac1_plugin.Tac2core_ltac1 *)
module Tac2core_ltac1 = Ltac2_ltac1_plugin__Tac2core_ltac1

(** @canonical Ltac2_ltac1_plugin.Tac2quote_ltac1 *)
module Tac2quote_ltac1 = Ltac2_ltac1_plugin__Tac2quote_ltac1

(** @canonical Ltac2_ltac1_plugin.Tac2stdlib_ltac1 *)
module Tac2stdlib_ltac1 = Ltac2_ltac1_plugin__Tac2stdlib_ltac1
