(* generated by dune *)

(** @canonical Coq_checklib.Analyze *)
module Analyze = Coq_checklib__Analyze

(** @canonical Coq_checklib.CheckFlags *)
module CheckFlags = Coq_checklib__CheckFlags

(** @canonical Coq_checklib.CheckInductive *)
module CheckInductive = Coq_checklib__CheckInductive

(** @canonical Coq_checklib.CheckLibrary *)
module CheckLibrary = Coq_checklib__CheckLibrary

(** @canonical Coq_checklib.Check_stat *)
module Check_stat = Coq_checklib__Check_stat

(** @canonical Coq_checklib.Coqchk_main *)
module Coqchk_main = Coq_checklib__Coqchk_main

(** @canonical Coq_checklib.Mod_checking *)
module Mod_checking = Coq_checklib__Mod_checking

(** @canonical Coq_checklib.Safe_checking *)
module Safe_checking = Coq_checklib__Safe_checking

(** @canonical Coq_checklib.Validate *)
module Validate = Coq_checklib__Validate

(** @canonical Coq_checklib.Values *)
module Values = Coq_checklib__Values
