(* DO NOT EDIT THIS FILE: automatically generated by ../configure *)
(* Exact command that generated this file: *)
(* ./_build/default/tools/configure/configure.exe -prefix /usr -mandir /usr/share/man -docdir /usr/share/doc -libdir /usr/lib/ocaml/coq *)

type relocatable_path = NotRelocatable of string | Relocatable of string
let install_prefix = NotRelocatable "/usr"
let coqlib = NotRelocatable "/usr/lib/ocaml/coq"
let configdir = NotRelocatable "/usr/etc/xdg/coq"
let datadir = NotRelocatable "/usr/share/coq"
let docdir = NotRelocatable "/usr/share/doc"
let coqlibsuffix = "lib/ocaml/coq"
let configdirsuffix = "etc/xdg/coq"
let datadirsuffix = "share/coq"
let docdirsuffix = "share/doc"
let ocamlfind = "/usr/bin/ocamlfind"
let caml_flags = "-thread -bin-annot -strict-sequence -w -a+1..3-4+5..8-9+10..26-27+28..39-40-41-42+43-44-45+46..47-48+49..57-58+59..66-67-68+69-70"
let version = "9.1.1"
let caml_version = "5.4.0"
let caml_version_nums = [5;4;0]
let arch = "Linux"
let arch_is_win32 = false
let exec_extension = ""
let has_natdynlink = true
let vo_version = 90100l
let browser = "xdg-open \"%s\""
let wwwcoq = "http://rocq-prover.org/"
let wwwbugtracker = "http://rocq-prover.org/bugs/"
let wwwrefman = "http://rocq-prover.org/doc/V9.1.1/refman/"
let wwwstdlib = "http://rocq-prover.org/doc/V9.1.1/stdlib/"
let bytecode_compiler = true
type native_compiler = NativeOff | NativeOn of { ondemand : bool }
let native_compiler = NativeOff

let core_src_dirs = [
  "boot";
  "config";
  "lib";
  "clib";
  "kernel";
  "library";
  "engine";
  "pretyping";
  "interp";
  "gramlib";
  "parsing";
  "proofs";
  "tactics";
  "toplevel";
  "printing";
  "ide";
  "stm";
  "vernac";
]

let plugins_dirs = [
  "plugins/btauto";
  "plugins/cc";
  "plugins/derive";
  "plugins/extraction";
  "plugins/firstorder";
  "plugins/funind";
  "plugins/ltac";
  "plugins/ltac2";
  "plugins/ltac2_ltac1";
  "plugins/micromega";
  "plugins/nsatz";
  "plugins/ring";
  "plugins/rtauto";
  "plugins/ssr";
  "plugins/ssrmatching";
  "plugins/syntax";
]

let all_src_dirs = core_src_dirs @ plugins_dirs
