(* generated by dune *)

(** @canonical Micromega_core_plugin.Micromega *)
module Micromega = Micromega_core_plugin__Micromega

(** @canonical Micromega_core_plugin.Mutils *)
module Mutils = Micromega_core_plugin__Mutils

(** @canonical Micromega_core_plugin.NumCompat *)
module NumCompat = Micromega_core_plugin__NumCompat

(** @canonical Micromega_core_plugin.Sos *)
module Sos = Micromega_core_plugin__Sos

(** @canonical Micromega_core_plugin.Sos_lib *)
module Sos_lib = Micromega_core_plugin__Sos_lib

(** @canonical Micromega_core_plugin.Sos_types *)
module Sos_types = Micromega_core_plugin__Sos_types
