(* generated by dune *)

(** @canonical Number_string_notation_plugin.G_number_string *)
module G_number_string = Number_string_notation_plugin__G_number_string

(** @canonical Number_string_notation_plugin.Number_string *)
module Number_string = Number_string_notation_plugin__Number_string
