(* DO NOT EDIT: automatically generated by kernel/genOpcodeFiles.ml *)
let opACC0 = 0

let opACC1 = 1

let opACC2 = 2

let opACC3 = 3

let opACC4 = 4

let opACC5 = 5

let opACC6 = 6

let opACC7 = 7

let opACC = 8

let opPUSH = 9

let opPUSHACC1 = 10

let opPUSHACC2 = 11

let opPUSHACC3 = 12

let opPUSHACC4 = 13

let opPUSHACC5 = 14

let opPUSHACC6 = 15

let opPUSHACC7 = 16

let opPUSHACC = 17

let opPUSHACCMANY = 18

let opPOP = 19

let opENVACC0 = 20

let opENVACC1 = 21

let opENVACC2 = 22

let opENVACC3 = 23

let opENVACC = 24

let opPUSHENVACC0 = 25

let opPUSHENVACC1 = 26

let opPUSHENVACC2 = 27

let opPUSHENVACC3 = 28

let opPUSHENVACC = 29

let opPUSHENVACCMANY = 30

let opPUSH_RETADDR = 31

let opAPPLY = 32

let opAPPLY1 = 33

let opAPPLY2 = 34

let opAPPLY3 = 35

let opAPPLY4 = 36

let opAPPTERM = 37

let opAPPTERM1 = 38

let opAPPTERM2 = 39

let opAPPTERM3 = 40

let opRETURN = 41

let opRESTART = 42

let opGRAB = 43

let opGRABREC = 44

let opCLOSURE = 45

let opCLOSUREREC = 46

let opCLOSURECOFIX = 47

let opOFFSETCLOSURE0 = 48

let opOFFSETCLOSURE1 = 49

let opOFFSETCLOSURE = 50

let opPUSHOFFSETCLOSURE0 = 51

let opPUSHOFFSETCLOSURE1 = 52

let opPUSHOFFSETCLOSURE = 53

let opGETGLOBAL = 54

let opPUSHGETGLOBAL = 55

let opMAKEBLOCK = 56

let opMAKEBLOCK1 = 57

let opMAKEBLOCK2 = 58

let opMAKEBLOCK3 = 59

let opMAKEBLOCK4 = 60

let opSWITCH = 61

let opPUSHFIELDS = 62

let opGETFIELD0 = 63

let opGETFIELD1 = 64

let opGETFIELD = 65

let opSETFIELD = 66

let opPROJ = 67

let opENSURESTACKCAPACITY = 68

let opCONST0 = 69

let opCONST1 = 70

let opCONST2 = 71

let opCONST3 = 72

let opCONSTINT = 73

let opPUSHCONST0 = 74

let opPUSHCONST1 = 75

let opPUSHCONST2 = 76

let opPUSHCONST3 = 77

let opPUSHCONSTINT = 78

let opACCUMULATE = 79

let opMAKESWITCHBLOCK = 80

let opMAKEACCU = 81

let opSUBSTINSTANCE = 82

let opBRANCH = 83

let opCHECKADDINT63 = 84

let opCHECKADDCINT63 = 85

let opCHECKADDCARRYCINT63 = 86

let opCHECKSUBINT63 = 87

let opCHECKSUBCINT63 = 88

let opCHECKSUBCARRYCINT63 = 89

let opCHECKMULINT63 = 90

let opCHECKMULCINT63 = 91

let opCHECKDIVINT63 = 92

let opCHECKMODINT63 = 93

let opCHECKDIVSINT63 = 94

let opCHECKMODSINT63 = 95

let opCHECKDIVEUCLINT63 = 96

let opCHECKDIV21INT63 = 97

let opCHECKLXORINT63 = 98

let opCHECKLORINT63 = 99

let opCHECKLANDINT63 = 100

let opCHECKLSLINT63 = 101

let opCHECKLSRINT63 = 102

let opCHECKASRINT63 = 103

let opCHECKADDMULDIVINT63 = 104

let opCHECKEQINT63 = 105

let opCHECKLTINT63 = 106

let opCHECKLEINT63 = 107

let opCHECKLTSINT63 = 108

let opCHECKLESINT63 = 109

let opCHECKCOMPAREINT63 = 110

let opCHECKCOMPARESINT63 = 111

let opCHECKHEAD0INT63 = 112

let opCHECKTAIL0INT63 = 113

let opCHECKOPPFLOAT = 114

let opCHECKABSFLOAT = 115

let opCHECKEQFLOAT = 116

let opCHECKLTFLOAT = 117

let opCHECKLEFLOAT = 118

let opCHECKCOMPAREFLOAT = 119

let opCHECKEQUALFLOAT = 120

let opCHECKCLASSIFYFLOAT = 121

let opCHECKADDFLOAT = 122

let opCHECKSUBFLOAT = 123

let opCHECKMULFLOAT = 124

let opCHECKDIVFLOAT = 125

let opCHECKSQRTFLOAT = 126

let opCHECKFLOATOFINT63 = 127

let opCHECKFLOATNORMFRMANTISSA = 128

let opCHECKFRSHIFTEXP = 129

let opCHECKLDSHIFTEXP = 130

let opCHECKNEXTUPFLOAT = 131

let opCHECKNEXTDOWNFLOAT = 132

let opCHECKNEXTUPFLOATINPLACE = 133

let opCHECKNEXTDOWNFLOATINPLACE = 134

let opCHECKCAMLCALL2_1 = 135

let opCHECKCAMLCALL1 = 136

let opCHECKCAMLCALL2 = 137

let opCHECKCAMLCALL3 = 138

let opCHECKCAMLCALL3_1 = 139

let opSTOP = 140

