(* generated by dune *)

(** @canonical Nsatz_core_plugin.Ideal *)
module Ideal = Nsatz_core_plugin__Ideal

(** @canonical Nsatz_core_plugin.Nsatz *)
module Nsatz = Nsatz_core_plugin__Nsatz

(** @canonical Nsatz_core_plugin.Polynom *)
module Polynom = Nsatz_core_plugin__Polynom

(** @canonical Nsatz_core_plugin.Utile *)
module Utile = Nsatz_core_plugin__Utile
